@@ -9,7 +9,10 @@ extern "C" {
 #endif
 
 #include "jdupes.h"
-extern void linkfiles(file_t *files, const int hard);
+/* Don't define the function if it's just empty anyway */
+#if !(defined NO_HARDLINKS && defined NO_SYMLINKS)
+extern void linkfiles(file_t *files, const int linktype, const int only_current);
+#endif
 
 #ifdef __cplusplus
 }
