@@ -1,4 +1,4 @@
-/* jdupes (C) 2015-2020 Jody Bruchon <jody@jodybruchon.com>
+/* jdupes (C) 2015-2021 Jody Bruchon <jody@jodybruchon.com>
    Forked from fdupes 1.51 (C) 1999-2014 Adrian Lopez
 
    Permission is hereby granted, free of charge, to any person
@@ -98,11 +98,12 @@ uint_fast32_t flags = 0, a_flags = 0, p_flags = 0;
 
 static const char *program_name;
 
-/* This gets used in many functions */
+/* Stat and SIGUSR */
 #ifdef ON_WINDOWS
  struct winstat s;
 #else
  struct stat s;
+ static int usr1_toggle = 0;
 #endif
 
 /* Larger chunk size makes large files process faster but uses more RAM */
@@ -297,8 +298,13 @@ void sighandler(const int signum)
 void sigusr1(const int signum)
 {
   (void)signum;
-  if (!ISFLAG(flags, F_SOFTABORT)) SETFLAG(flags, F_SOFTABORT);
-  else CLEARFLAG(flags, F_SOFTABORT);
+  if (!ISFLAG(flags, F_SOFTABORT)) {
+    SETFLAG(flags, F_SOFTABORT);
+    usr1_toggle = 1;
+  } else {
+    CLEARFLAG(flags, F_SOFTABORT);
+    usr1_toggle = 2;
+  }
   return;
 }
 #endif
@@ -399,6 +405,13 @@ static void update_progress(const char * const restrict msg, const int file_perc
     fflush(stderr);
   }
   time1.tv_sec = time2.tv_sec;
+#ifndef ON_WINDOWS
+  /* Notify of change to soft abort status if SIGUSR1 received */
+  if (usr1_toggle != 0) {
+    fprintf(stderr, "\njdupes received a USR1 signal; soft abort (-Z) is now %s\n", usr1_toggle == 1 ? "ON" : "OFF" );
+    usr1_toggle = 0;
+  }
+#endif
   return;
 }
 
@@ -468,7 +481,7 @@ skip_empty:
  * Returns 1 if changed, 0 if not changed, negative if error */
 extern int file_has_changed(file_t * const restrict file)
 {
-  /* If -t/--nochangecheck specified then completely bypass this code */
+  /* If -t/--no-change-check specified then completely bypass this code */
   if (ISFLAG(flags, F_NOCHANGECHECK)) return 0;
 
   if (file == NULL || file->d_name == NULL) nullptr("file_has_changed()");
@@ -651,7 +664,7 @@ extern int getdirstats(const char * const restrict name,
  * -2 on an absolute exclusion condition met
  *  2 on an absolute match condition met
  * -3 on exclusion due to isolation
- * -4 on exlusion due to same filesystem
+ * -4 on exclusion due to same filesystem
  * -5 on exclusion due to permissions */
 extern int check_conditions(const file_t * const restrict file1, const file_t * const restrict file2)
 {
@@ -1357,7 +1370,7 @@ static file_t **checkmatch(filetree_t * restrict tree, file_t * const restrict f
 
     /* Print partial hash matching pairs if requested */
     if (cmpresult == 0 && ISFLAG(p_flags, PF_PARTIAL))
-      printf("Partial hashes match:\n   %s\n   %s\n\n", file->d_name, tree->file->d_name);
+      printf("\nPartial hashes match:\n   %s\n   %s\n\n", file->d_name, tree->file->d_name);
 
     if (file->size <= PARTIAL_HASH_SIZE || ISFLAG(flags, F_PARTIALONLY)) {
       if (ISFLAG(flags, F_PARTIALONLY)) { LOUD(fprintf(stderr, "checkmatch: partial only mode: treating partial hash as full hash\n")); }
@@ -1627,13 +1640,13 @@ static inline void help_text(void)
 #ifdef LOUD
   printf(" -@ --loud        \toutput annoying low-level debug info while running\n");
 #endif
-  printf(" -0 --printnull   \toutput nulls instead of CR/LF (like 'find -print0')\n");
-  printf(" -1 --one-file-system \tdo not match files on different filesystems/devices\n");
-  printf(" -A --nohidden    \texclude hidden files from consideration\n");
+  printf(" -0 --print-null  \toutput nulls instead of CR/LF (like 'find -print0')\n");
+  printf(" -1 --one-file-system\tdo not match files on different filesystems/devices\n");
+  printf(" -A --no-hidden    \texclude hidden files from consideration\n");
 #ifdef ENABLE_DEDUPE
   printf(" -B --dedupe      \tdo a copy-on-write (reflink/clone) deduplication\n");
 #endif
-  printf(" -C --chunksize=# \toverride I/O chunk size (min %d, max %d)\n", MIN_CHUNK_SIZE, MAX_CHUNK_SIZE);
+  printf(" -C --chunk-size=#\toverride I/O chunk size (min %d, max %d)\n", MIN_CHUNK_SIZE, MAX_CHUNK_SIZE);
   printf(" -d --delete      \tprompt user for files to preserve and delete all\n");
   printf("                  \tothers; important: under particular circumstances,\n");
   printf("                  \tdata may be lost when using this option together\n");
@@ -1643,10 +1656,10 @@ static inline void help_text(void)
 #ifdef DEBUG
   printf(" -D --debug       \toutput debug statistics after completion\n");
 #endif
-  printf(" -f --omitfirst   \tomit the first file in each set of matches\n");
+  printf(" -f --omit-first  \tomit the first file in each set of matches\n");
   printf(" -h --help        \tdisplay this help message\n");
 #ifndef NO_HARDLINKS
-  printf(" -H --hardlinks   \ttreat any linked files as duplicate files. Normally\n");
+  printf(" -H --hard-links  \ttreat any linked files as duplicate files. Normally\n");
   printf("                  \tlinked files are treated as non-duplicates for safety\n");
 #endif
   printf(" -i --reverse     \treverse (invert) the match sort order\n");
@@ -1654,13 +1667,13 @@ static inline void help_text(void)
   printf(" -I --isolate     \tfiles in the same specified directory won't match\n");
 #endif
   printf(" -j --json        \tproduce JSON (machine-readable) output\n");
-/*  printf(" -K --skiphash    \tskip full file hashing (may be faster; 100%% safe)\n"); */
-  printf("                  \tWARNING: in development, not fully working yet!\n");
+/*  printf(" -K --skip-hash   \tskip full file hashing (may be faster; 100%% safe)\n");
+    printf("                  \tWARNING: in development, not fully working yet!\n"); */
 #ifndef NO_SYMLINKS
-  printf(" -l --linksoft    \tmake relative symlinks for duplicates w/o prompting\n");
+  printf(" -l --link-soft    \tmake relative symlinks for duplicates w/o prompting\n");
 #endif
 #ifndef NO_HARDLINKS
-  printf(" -L --linkhard    \thard link all duplicate files without prompting\n");
+  printf(" -L --link-hard    \thard link all duplicate files without prompting\n");
  #ifdef ON_WINDOWS
   printf("                  \tWindows allows a maximum of 1023 hard links per file;\n");
   printf("                  \tlinking large match sets will result in multiple sets\n");
@@ -1668,14 +1681,14 @@ static inline void help_text(void)
  #endif /* ON_WINDOWS */
 #endif /* NO_HARDLINKS */
   printf(" -m --summarize   \tsummarize dupe information\n");
-  printf(" -M --printwithsummary\twill print matches and --summarize at the end\n");
-  printf(" -N --noprompt    \ttogether with --delete, preserve the first file in\n");
+  printf(" -M --print-summarize\tprint match sets and --summarize at the end\n");
+  printf(" -N --no-prompt   \ttogether with --delete, preserve the first file in\n");
   printf("                  \teach set of duplicates and delete the rest without\n");
   printf("                  \tprompting the user\n");
   printf(" -o --order=BY    \tselect sort order for output, linking and deleting; by\n");
   printf("                  \tmtime (BY=time) or filename (BY=name, the default)\n");
 #ifndef NO_USER_ORDER
-  printf(" -O --paramorder  \tParameter order is more important than selected -o sort\n");
+  printf(" -O --param-order  \tParameter order is more important than selected -o sort\n");
 #endif
 #ifndef NO_PERMS
   printf(" -p --permissions \tdon't consider files with different owner/group or\n");
@@ -1693,18 +1706,18 @@ static inline void help_text(void)
   printf(" -s --symlinks    \tfollow symlinks\n");
 #endif
   printf(" -S --size        \tshow size of duplicate files\n");
-  printf(" -t --nochangecheck\tdisable security check for file changes (aka TOCTTOU)\n");
+  printf(" -t --no-change-check\tdisable security check for file changes (aka TOCTTOU)\n");
   printf(" -T --partial-only \tmatch based on partial hashes only. WARNING:\n");
   printf("                  \tEXTREMELY DANGEROUS paired with destructive actions!\n");
   printf("                  \t-T must be specified twice to work. Read the manual!\n");
-  printf(" -u --printunique \tprint only a list of unique (non-matched) files\n");
-  printf(" -U --notravcheck \tdisable double-traversal safety check (BE VERY CAREFUL)\n");
+  printf(" -u --print-unique\tprint only a list of unique (non-matched) files\n");
+  printf(" -U --no-trav-check\tdisable double-traversal safety check (BE VERY CAREFUL)\n");
   printf("                  \tThis fixes a Google Drive File Stream recursion issue\n");
   printf(" -v --version     \tdisplay jdupes version and license information\n");
-  printf(" -X --extfilter=x:y\tfilter files based on specified criteria\n");
+  printf(" -X --ext-filter=x:y\tfilter files based on specified criteria\n");
   printf("                  \tUse '-X help' for detailed extfilter help\n");
-  printf(" -z --zeromatch   \tconsider zero-length files to be duplicates\n");
-  printf(" -Z --softabort   \tIf the user aborts (i.e. CTRL-C) act on matches so far\n");
+  printf(" -z --zero-match  \tconsider zero-length files to be duplicates\n");
+  printf(" -Z --soft-abort  \tIf the user aborts (i.e. CTRL-C) act on matches so far\n");
 #ifndef ON_WINDOWS
   printf("                  \tYou can send SIGUSR1 to the program to toggle this\n");
 #endif
@@ -1716,7 +1729,7 @@ static inline void help_text(void)
 
 static void help_text_extfilter(void)
 {
-  printf("Detailed help for jdupes -X/--extfilter options\n");
+  printf("Detailed help for jdupes -X/--ext-filter options\n");
   printf("General format: jdupes -X filter[:value][size_suffix]\n\n");
 
   /* FIXME: Remove after v1.19.0 */
@@ -1778,27 +1791,36 @@ int main(int argc, char **argv)
   static const struct option long_options[] =
   {
     { "loud", 0, 0, '@' },
-    { "printnull", 0, 0, '0' },
+    { "printnull", 0, 0, '0' }, //LEGACY
+    { "print-null", 0, 0, '0' },
     { "one-file-system", 0, 0, '1' },
-    { "nohidden", 0, 0, 'A' },
+    { "nohidden", 0, 0, 'A' }, //LEGACY
+    { "no-hidden", 0, 0, 'A' },
     { "dedupe", 0, 0, 'B' },
-    { "chunksize", 1, 0, 'C' },
+    { "chunksize", 1, 0, 'C' }, //LEGACY
+    { "chunk-size", 1, 0, 'C' },
     { "debug", 0, 0, 'D' },
     { "delete", 0, 0, 'd' },
-    { "omitfirst", 0, 0, 'f' },
-    { "hardlinks", 0, 0, 'H' },
+    { "omitfirst", 0, 0, 'f' }, //LEGACY
+    { "omit-first", 0, 0, 'f' },
+    { "hardlinks", 0, 0, 'H' }, //LEGACY
+    { "hard-links", 0, 0, 'H' },
     { "help", 0, 0, 'h' },
     { "isolate", 0, 0, 'I' },
     { "reverse", 0, 0, 'i' },
     { "json", 0, 0, 'j' },
-    { "skiphash", 0, 0, 'K' },
-    { "linkhard", 0, 0, 'L' },
-    { "linksoft", 0, 0, 'l' },
-    { "printwithsummary", 0, 0, 'M'},
+/*    { "skip-hash", 0, 0, 'K' }, */
+    { "linkhard", 0, 0, 'L' }, //LEGACY
+    { "link-hard", 0, 0, 'L' },
+    { "linksoft", 0, 0, 'l' }, //LEGACY
+    { "link-soft", 0, 0, 'l' },
+    { "printwithsummary", 0, 0, 'M'}, //LEGACY
+    { "print-summarize", 0, 0, 'M'},
     { "summarize", 0, 0, 'm'},
-    { "noprompt", 0, 0, 'N' },
-    { "noempty", 0, 0, 'n' },
-    { "paramorder", 0, 0, 'O' },
+    { "noprompt", 0, 0, 'N' }, //LEGACY
+    { "no-prompt", 0, 0, 'N' },
+    { "paramorder", 0, 0, 'O' }, //LEGACY
+    { "param-order", 0, 0, 'O' },
     { "order", 1, 0, 'o' },
     { "print", 1, 0, 'P' },
     { "permissions", 0, 0, 'p' },
@@ -1809,13 +1831,19 @@ int main(int argc, char **argv)
     { "size", 0, 0, 'S' },
     { "symlinks", 0, 0, 's' },
     { "partial-only", 0, 0, 'T' },
-    { "nochangecheck", 0, 0, 't' },
-    { "notravcheck", 0, 0, 'U' },
-    { "printunique", 0, 0, 'u' },
+    { "nochangecheck", 0, 0, 't' }, //LEGACY
+    { "no-change-check", 0, 0, 't' },
+    { "notravcheck", 0, 0, 'U' }, //LEGACY
+    { "no-trav-check", 0, 0, 'U' },
+    { "printunique", 0, 0, 'u' }, //LEGACY
+    { "print-unique", 0, 0, 'u' },
     { "version", 0, 0, 'v' },
-    { "extfilter", 1, 0, 'X' },
-    { "softabort", 0, 0, 'Z' },
-    { "zeromatch", 0, 0, 'z' },
+    { "extfilter", 1, 0, 'X' }, //LEGACY
+    { "ext-filter", 1, 0, 'X' },
+    { "softabort", 0, 0, 'Z' }, //LEGACY
+    { "soft-abort", 0, 0, 'Z' },
+    { "zeromatch", 0, 0, 'z' }, //LEGACY
+    { "zero-match", 0, 0, 'z' },
     { NULL, 0, 0, 0 }
   };
 #define GETOPT getopt_long
@@ -1879,11 +1907,11 @@ int main(int argc, char **argv)
     switch (opt) {
     case '0':
       SETFLAG(a_flags, FA_PRINTNULL);
-      LOUD(fprintf(stderr, "opt: print null instead of newline (--printnull)\n");)
+      LOUD(fprintf(stderr, "opt: print null instead of newline (--print-null)\n");)
       break;
     case '1':
       SETFLAG(flags, F_ONEFS);
-      LOUD(fprintf(stderr, "opt: recursion across filesystems disabled (--onefs)\n");)
+      LOUD(fprintf(stderr, "opt: recursion across filesystems disabled (--one-file-system)\n");)
       break;
     case 'A':
       SETFLAG(flags, F_EXCLUDEHIDDEN);
@@ -1899,7 +1927,7 @@ int main(int argc, char **argv)
       break;
     case 'd':
       SETFLAG(a_flags, FA_DELETEFILES);
-      LOUD(fprintf(stderr, "opt: delete files after matching (--deletefiles)\n");)
+      LOUD(fprintf(stderr, "opt: delete files after matching (--delete)\n");)
       break;
     case 'D':
 #ifdef DEBUG
@@ -1908,7 +1936,7 @@ int main(int argc, char **argv)
       break;
     case 'f':
       SETFLAG(a_flags, FA_OMITFIRST);
-      LOUD(fprintf(stderr, "opt: omit first match from each match set (--omitfirst)\n");)
+      LOUD(fprintf(stderr, "opt: omit first match from each match set (--omit-first)\n");)
       break;
     case 'h':
       help_text();
@@ -1917,11 +1945,11 @@ int main(int argc, char **argv)
 #ifndef NO_HARDLINKS
     case 'H':
       SETFLAG(flags, F_CONSIDERHARDLINKS);
-      LOUD(fprintf(stderr, "opt: hard links count as matches (--hardlinks)\n");)
+      LOUD(fprintf(stderr, "opt: hard links count as matches (--hard-links)\n");)
       break;
     case 'L':
       SETFLAG(a_flags, FA_HARDLINKFILES);
-      LOUD(fprintf(stderr, "opt: convert duplicates to hard links (--linkhard)\n");)
+      LOUD(fprintf(stderr, "opt: convert duplicates to hard links (--link-hard)\n");)
       break;
 #endif
     case 'i':
@@ -1935,7 +1963,7 @@ int main(int argc, char **argv)
       break;
     case 'O':
       SETFLAG(flags, F_USEPARAMORDER);
-      LOUD(fprintf(stderr, "opt: parameter order takes precedence (--paramorder)\n");)
+      LOUD(fprintf(stderr, "opt: parameter order takes precedence (--param-order)\n");)
       break;
 #else
     case 'I':
@@ -1945,7 +1973,7 @@ int main(int argc, char **argv)
 #endif
     case 'j':
       SETFLAG(a_flags, FA_PRINTJSON);
-      LOUD(fprintf(stderr, "opt: print output in JSON format (--printjson)\n");)
+      LOUD(fprintf(stderr, "opt: print output in JSON format (--print-json)\n");)
       break;
     case 'K':
       SETFLAG(flags, F_SKIPHASH);
@@ -1957,10 +1985,7 @@ int main(int argc, char **argv)
     case 'M':
       SETFLAG(a_flags, FA_SUMMARIZEMATCHES);
       SETFLAG(a_flags, FA_PRINTMATCHES);
-      LOUD(fprintf(stderr, "opt: print matches with a summary (--printwithsummary)\n");)
-      break;
-    case 'n':
-      //fprintf(stderr, "note: -n/--noempty is the default behavior now and is deprecated.\n");
+      LOUD(fprintf(stderr, "opt: print matches with a summary (--print-summarize)\n");)
       break;
     case 'N':
       SETFLAG(flags, F_NOPROMPT);
@@ -1994,33 +2019,33 @@ int main(int argc, char **argv)
       break;
     case 'R':
       SETFLAG(flags, F_RECURSEAFTER);
-      LOUD(fprintf(stderr, "opt: partial recursion enabled (--recurseafter)\n");)
+      LOUD(fprintf(stderr, "opt: partial recursion enabled (--recurse-after)\n");)
       break;
     case 't':
       SETFLAG(flags, F_NOCHANGECHECK);
-      LOUD(fprintf(stderr, "opt: TOCTTOU safety check disabled (--nochangecheck)\n");)
+      LOUD(fprintf(stderr, "opt: TOCTTOU safety check disabled (--no-change-check)\n");)
       break;
     case 'T':
       if (partialonly_spec == 0)
         partialonly_spec = 1;
       else {
         partialonly_spec = 2;
-        fprintf(stderr, "\nBIG FAT WARNING: -T/--partialonly is EXTREMELY DANGEROUS! Read the manual!\n\n");
+        fprintf(stderr, "\nBIG FAT WARNING: -T/--partial-only is EXTREMELY DANGEROUS! Read the manual!\n\n");
         SETFLAG(flags, F_PARTIALONLY);
       }
       break;
     case 'u':
       SETFLAG(a_flags, FA_PRINTUNIQUE);
-      LOUD(fprintf(stderr, "opt: print only non-matched (unique) files (--printunique)\n");)
+      LOUD(fprintf(stderr, "opt: print only non-matched (unique) files (--print-unique)\n");)
       break;
     case 'U':
       SETFLAG(flags, F_NOTRAVCHECK);
-      LOUD(fprintf(stderr, "opt: double-traversal safety check disabled (--notravcheck)\n");)
+      LOUD(fprintf(stderr, "opt: double-traversal safety check disabled (--no-trav-check)\n");)
       break;
 #ifndef NO_SYMLINKS
     case 'l':
       SETFLAG(a_flags, FA_MAKESYMLINKS);
-      LOUD(fprintf(stderr, "opt: convert duplicates to symbolic links (--linksoft)\n");)
+      LOUD(fprintf(stderr, "opt: convert duplicates to symbolic links (--link-soft)\n");)
       break;
     case 's':
       SETFLAG(flags, F_FOLLOWLINKS);
@@ -2036,11 +2061,11 @@ int main(int argc, char **argv)
       break;
     case 'z':
       SETFLAG(flags, F_INCLUDEEMPTY);
-      LOUD(fprintf(stderr, "opt: zero-length files count as matches (--zeromatch)\n");)
+      LOUD(fprintf(stderr, "opt: zero-length files count as matches (--zero-match)\n");)
       break;
     case 'Z':
       SETFLAG(flags, F_SOFTABORT);
-      LOUD(fprintf(stderr, "opt: soft-abort mode enabled (--softabort)\n");)
+      LOUD(fprintf(stderr, "opt: soft-abort mode enabled (--soft-abort)\n");)
       break;
     case '@':
 #ifdef LOUD_DEBUG
@@ -2070,7 +2095,7 @@ int main(int argc, char **argv)
           c++;
         }
       } else printf(" none");
-      printf("\nCopyright (C) 2015-2020 by Jody Bruchon and contributors\n");
+      printf("\nCopyright (C) 2015-2021 by Jody Bruchon and contributors\n");
       printf("Forked from fdupes 1.51, (C) 1999-2014 Adrian Lopez and contributors\n\n");
       printf("Permission is hereby granted, free of charge, to any person obtaining a copy of\n");
       printf("this software and associated documentation files (the \"Software\"), to deal in\n");
@@ -2089,10 +2114,8 @@ int main(int argc, char **argv)
       printf("OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\n");
       printf("SOFTWARE.\n");
       printf("\nIf you find this software useful, please consider financially supporting\n");
-      printf("its continued development by donating to the author's SubscribeStar:\n");
-      printf("          https://SubscribeStar.com/JodyBruchon\n");
-      printf("\nNew releases, bug fixes, and more at the jdupes GitHub project page:\n");
-      printf("             https://github.com/jbruchon/jdupes\n");
+      printf("its development through the author's home page: https://www.jodybruchon.com/\n");
+      printf("\nReport bugs and get the latest releases: https://github.com/jbruchon/jdupes\n");
       exit(EXIT_SUCCESS);
     case 'o':
       if (!strncasecmp("name", optarg, 5)) {
@@ -2181,12 +2204,17 @@ int main(int argc, char **argv)
       !!ISFLAG(a_flags, FA_DEDUPEFILES);
 
   if (pm > 1) {
-      fprintf(stderr, "Only one of --summarize, --printwithsummary, --delete, --linkhard,\n--linksoft, --json, or --dedupe may be used\n");
+      fprintf(stderr, "Only one of --summarize, --print-summarize, --delete, --link-hard,\n--link-soft, --json, or --dedupe may be used\n");
       string_malloc_destroy();
       exit(EXIT_FAILURE);
   }
   if (pm == 0) SETFLAG(a_flags, FA_PRINTMATCHES);
 
+#ifndef ON_WINDOWS
+  /* Catch SIGUSR1 and use it to enable -Z */
+  signal(SIGUSR1, sigusr1);
+#endif
+
   if (ISFLAG(flags, F_RECURSEAFTER)) {
     firstrecurse = nonoptafter("--recurse:", argc, oldargv, argv);
 
@@ -2238,10 +2266,6 @@ int main(int argc, char **argv)
 
   /* Catch CTRL-C */
   signal(SIGINT, sighandler);
-#ifndef ON_WINDOWS
-  /* Catch SIGUSR1 and use it to enable -Z */
-  signal(SIGUSR1, sigusr1);
-#endif
 
   while (curfile) {
     static file_t **match = NULL;
@@ -2330,10 +2354,10 @@ skip_file_scan:
     else deletefiles(files, 1, stdin);
   }
 #ifndef NO_SYMLINKS
-  if (ISFLAG(a_flags, FA_MAKESYMLINKS)) linkfiles(files, 0);
+  if (ISFLAG(a_flags, FA_MAKESYMLINKS)) linkfiles(files, 0, 0);
 #endif
 #ifndef NO_HARDLINKS
-  if (ISFLAG(a_flags, FA_HARDLINKFILES)) linkfiles(files, 1);
+  if (ISFLAG(a_flags, FA_HARDLINKFILES)) linkfiles(files, 1, 0);
 #endif /* NO_HARDLINKS */
 #ifdef ENABLE_DEDUPE
   if (ISFLAG(a_flags, FA_DEDUPEFILES)) dedupefiles(files);
