@@ -1,6 +1,6 @@
 /* Jody Bruchon's sorting code library
  *
- * Copyright (C) 2014-2020 by Jody Bruchon <jody@jodybruchon.com>
+ * Copyright (C) 2014-2021 by Jody Bruchon <jody@jodybruchon.com>
  * Released under The MIT License
  */
 
@@ -10,18 +10,19 @@
 #define IS_NUM(a) (((a >= '0') && (a <= '9')) ? 1 : 0)
 #define IS_LOWER(a) (((a >= 'a') && (a <= 'z')) ? 1 : 0)
 
-extern int numeric_sort(const char * restrict c1,
-                const char * restrict c2, int sort_direction)
+extern int numeric_sort(char * restrict c1,
+                char * restrict c2, int sort_direction)
 {
   int len1 = 0, len2 = 0;
-  int precompare = 0;
+  int precompare;
+  char *rewind1, *rewind2;
 
   if (c1 == NULL || c2 == NULL) return -99;
 
   /* Numerically correct sort */
   while (*c1 != '\0' && *c2 != '\0') {
-    /* Reset string length counters */
-    len1 = 0; len2 = 0;
+    /* Reset string length counters and rewind points */
+    len1 = 0; len2 = 0; rewind1 = c1; rewind2 = c2;
 
     /* Skip all sequences of zeroes */
     while (*c1 == '0') {
@@ -66,6 +67,10 @@ extern int numeric_sort(const char * restrict c1,
        * of equal length. Use the precompare result
        * as the result for this number comparison. */
       if (precompare != 0) return precompare;
+    } else {
+      /* Zeroes aren't followed by a digit; rewind the streams */
+      c1 = rewind1; c2 = rewind2;
+      len1 = 0; len2 = 0;
     }
 
     /* Do normal comparison */
