@@ -8,14 +8,23 @@
 extern "C" {
 #endif
 
-#include "jdupes.h"
+#include <stdio.h>
+
+extern int fwprint(FILE * const restrict stream, const char * const restrict str, const int cr);
 
 #ifdef UNICODE
-extern void slash_convert(char *path);
-extern void widearg_to_argv(int argc, wchar_t **wargv, char **argv);
-extern int fwprint(FILE * const restrict stream, const char * const restrict str, const int cr);
+ #ifndef WPATH_MAX
+  #define WPATH_MAX 8192
+ #endif
+ #ifndef M2W
+  #define M2W(a,b) MultiByteToWideChar(CP_UTF8, 0, a, -1, (LPWSTR)b, WPATH_MAX)
+ #endif
+ #ifndef W2M
+  #define W2M(a,b) WideCharToMultiByte(CP_UTF8, 0, a, -1, (LPSTR)b, WPATH_MAX, NULL, NULL)
+ #endif
+ extern void slash_convert(char *path);
+ extern void widearg_to_argv(int argc, wchar_t **wargv, char **argv);
 #else
- #define fwprint(a,b,c) fprintf(a, "%s%s", b, c ? "\n" : "")
  #define slash_convert(a)
 #endif /* UNICODE */
 
